#include "libgccvb.h"
#include "bgadjustment.h"
#include "chadjustment.h"
#include "bgbounce.h"
#include "chbounce.h"
#include "bgtitle.h"
#include "chtitle.h"
#include "level1.h"

const BYTE gravity[] = {4,3,4,3,3,4,3,3,4,3,3,3,2,2,3,2,2,3,2,2,2,3,2,2,2,3,2,2,2,2,2,3,2,2,2,2,2,1,2,2,1,2,2,1,2,1,2,1,1,2,1,1,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,1,0};
const BYTE h_acc[] = {0,0,0,1,1,1,1,2,2,2,3,3,4};

#define MAX_GRV (sizeof(gravity)-1)
#define MAX_ACC (sizeof(h_acc)-1)

#define LEVEL_LEN (sizeof(level1)/48)

#define UP_GRV 16
#define SCRL_THRSH 100

#define WRLD_SCORE 26
#define WRLD_OVRLY 27
#define WRLD_BILL 28
#define WRLD_PLYFR 29
#define WRLD_PLYBK 30
#define WRLD_BKGND 31
#define WRLD_LAST 25

//compo screen
void pvb_adjustment()
{
    int brightness, wait, i;

    //SETUP
    copymem((void*)CharSeg0, (void*)CHADJUSTMENT, 256*16);
    copymem((void*)BGMap(0), (void*)BGADJUSTMENT, 512*16);

    vbSetWorld(31, WRLD_LON|WRLD_OVR, 0, 0, 0, 0, 0, 0, 384, 224);
    vbSetWorld(30, WRLD_RON|WRLD_OVR, 0, 0, 0, 0, 0, 224, 384, 224);
    vbSetWorld(29, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);

    //FADE IN
    brightness = 0;
    while(brightness != 32)
    {
        for (i = 0; i <= 40; i++) 
        {
            wait = VIP_REGS[XPSTTS];
            while (wait == VIP_REGS[XPSTTS]);
        }
        VIP_REGS[BRTA] = brightness;
        VIP_REGS[BRTB] = brightness*2;
        VIP_REGS[BRTC] = brightness;
        brightness++;
    } 

    //WAIT FOR USER INPUT
    while (!(vbReadPad()&(K_ANY)));
  
    //FADE OUT
    brightness = 32;
    while(brightness != 0)
    {
        for (i = 0; i <= 40; i++) 
        {
            wait = VIP_REGS[XPSTTS];
            while (wait == VIP_REGS[XPSTTS]);
        }
        VIP_REGS[BRTA] = brightness;
        VIP_REGS[BRTB] = brightness*2;
        VIP_REGS[BRTC] = brightness;
        brightness--;
    } 
}

//title screen
void title_screen()
{
    int brightness, wait, i;

    //SETUP
    copymem((void*)CharSeg0, (void*)chtitle, 316*16);
    copymem((void*)BGMap(0), (void*)bgtitle, 1024*8);

    vbSetWorld(31, WRLD_ON, 0, 0, 0, 0, 0, 0, 384, 42); //name
    vbSetWorld(30, WRLD_ON, 112, -10, 44, 112, 0, 44, 148, 148); //smiley
    vbSetWorld(29, WRLD_ON, 100, -5, 190, 100, 0, 190, 180, 20); //press start
    vbSetWorld(28, WRLD_ON, 284, 4, 130, 284, 0, 130, 100, 100); //starring
    vbSetWorld(27, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);

    //FADE IN
    brightness = 0;
    while(brightness != 32)
    {
        for (i = 0; i <= 40; i++) 
        {
            wait = VIP_REGS[XPSTTS];
            while (wait == VIP_REGS[XPSTTS]);
        }
        VIP_REGS[BRTA] = brightness;
        VIP_REGS[BRTB] = brightness*2;
        VIP_REGS[BRTC] = brightness;
        brightness++;
    } 

    //WAIT FOR USER INPUT
    while (!(vbReadPad()&(K_STA)));
  
    //FADE OUT
    brightness = 32;
    while(brightness != 0)
    {
        for (i = 0; i <= 40; i++) 
        {
            wait = VIP_REGS[XPSTTS];
            while (wait == VIP_REGS[XPSTTS]);
        }
        VIP_REGS[BRTA] = brightness;
        VIP_REGS[BRTB] = brightness*2;
        VIP_REGS[BRTC] = brightness;
        brightness--;
    } 
}

int main()
{
  int i, j, objcnt;
	int jumpcnt;
	int jumpdir; //U=1, D=0
	int movecnt;
	int movedir; //L=0, R=1
	int scrollcnt;
	int x; //horizontal center point
	int y; //bottom
	int xtile;
	int ytile;
	int score;
	
	for (i=0; i<32; i++)
		WAVEDATA1[i*4]=i*4;
	
  vbDisplayOn();

	pvb_adjustment();

  title_screen();

	copymem((void*)(CharSeg3+(384*16)), (void*)chAsciiMap, 128*16); //Copy chars to Char segment 3
	copymem((void*) CharSeg0, (void*) chbouncy, 17*16); // Copy the chars to CharSeg0
	copymem((void*)BGMap(0), (void*) bgbouncy, 1024*8);
	sethword((void*)BGMap(4), (2048-128), 8192/2); //clear screen w/ blank char
	
	//vbSetWorld (World, Header, Screen X, Screen P, Screen Y, Buffer X, Buffer P, Buffer Y, Width, Height)
	vbSetWorld(WRLD_SCORE, WRLD_ON|4, 0, -10, 0, 0, 0, 0, 383, 223); //score
	vbSetWorld(WRLD_OVRLY, 0, 0, 0, 0, 0, 0, 0, 0, 0); //overlay
	vbSetWorld(WRLD_BILL, WRLD_ON, x, 0, 200-y, 0, 0, 0, 23, 23); //Bill
	vbSetWorld(WRLD_PLYFR, (WRLD_ON|WRLD_OBJ), 0, 0, 0, 0, 0, 0, 0, 0); //foreground playfield
	vbSetWorld(WRLD_PLYBK, 0, 0, 0, 0, 0, 0, 0, 0, 0); //background playfield (not used yet)
	vbSetWorld(WRLD_BKGND, WRLD_ON, 0, 10, 0, 0, 0, 136, 383, 223); //background
	vbSetWorld(WRLD_LAST, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0); // Blank world and END bit set 
	
	while (1)
	{
    jumpcnt = UP_GRV;
    jumpdir = 1;
    movecnt = 0;
    movedir = 0;
    scrollcnt=0;
    x = 192;
    y = 8;
    WA[WRLD_BILL].mx=0;
    score=0;
    
    objcnt=0;
    for (i=0; i<30; i++)
    {
      for (j=0; j<48; j++)
      {
        if (level1[i][j]==1)
          vbSetObject(1023-objcnt++, OBJ_ON, j*8, 0, 216-(i*8), 13);
      }
    }
    VIP_REGS[SPT3] = 1023;
    VIP_REGS[SPT2] = 1023-objcnt;
    
    cls(4,16);
    bgmap_print(4, 40, 2, itoa(score,10,1), 0);

    SSTOP=0;
    SND_REGS[WAVE1].SxRAM = 0;
		SND_REGS[WAVE1].SxLRV = 0x66;
		SND_REGS[WAVE1].SxEV0 = 0xF0;
		SND_REGS[WAVE1].SxFQL=(jumpcnt*16)&0xff;
    SND_REGS[WAVE1].SxFQH=((jumpcnt*16)>>8)+1;
		SND_REGS[WAVE1].SxINT=0x80; //if you do this every time, makes sound choppier (speech sounds better, but won't be pure tones)
		
		vbSetWorld(WRLD_BKGND, WRLD_ON, 0, 10, 0, 0, 0, 136, 383, 223); //background
		vbSetWorld(WRLD_PLYBK, 0, 0, 0, 0, 0, 0, 0, 0, 0); //background playfield (not used yet)
		
		vbDisplayShow();
    
    while (1)
    {
      if (jumpdir==1) //going up
      {
        if (jumpcnt<UP_GRV)
          jumpcnt=UP_GRV;
        jumpcnt++;
        y+=gravity[jumpcnt];
        WA[WRLD_BILL].mx=0;
        if (jumpcnt==MAX_GRV)
        {
          jumpdir=0;
          WA[WRLD_BILL].mx=24;
        }
      }
      else //going down
      {
        jumpcnt--;
        y-=gravity[jumpcnt];
        if (jumpcnt<0)
          VIP_REGS[BKCOL]=3;
        xtile=(x+4)/8;
        ytile=(y-8+scrollcnt+4)/8;
        if (level1[ytile][xtile]==0)
        {
          WA[WRLD_BILL].mx=48;
        }
        else if (y>0)
        {
          jumpdir=1;
          jumpcnt=UP_GRV;
          y=(ytile*8)-scrollcnt+8;
          WA[WRLD_BILL].mx=24;
        }
      }
      SND_REGS[WAVE1].SxFQL=(jumpcnt*16)&0xff;
      SND_REGS[WAVE1].SxFQH=((jumpcnt*16)>>8)+1;
        
      if (vbReadPad() & K_LL)
      {
        if (movedir==1) //going right
        {
          if (movecnt>0)
            movecnt--;
          else
          {
            movedir^=1;
            movecnt++;
          }
        }
        else
        {
          if (movecnt<MAX_ACC)
            movecnt++;
        }
      }
      else if (vbReadPad() & K_LR)
      {
        if (movedir==0) //going left
        {
          if (movecnt>0)
            movecnt--;
          else
          {
            movedir^=1;
            movecnt++;
          }
        }
        else
        {
          if (movecnt<MAX_ACC)
            movecnt++;
        }
      }
      
      if (movedir==1)
      {
        x+=h_acc[movecnt];
        if ((x+12)>383)
        {
          movedir^=1;
          x=383-((x+12)-383)-12;
        }
      }
      else
      {
        x-=h_acc[movecnt];
        if ((x-12)<0)
        {
          movedir^=1;
          x=-(x-12)+12;
        }
      }
      
      if (y>SCRL_THRSH)
      {
        scrollcnt+=y-SCRL_THRSH;
        y-=(y-SCRL_THRSH);
        //redraw screen here
        objcnt=0;
        for (i=(scrollcnt/8); i<(scrollcnt/8)+30; i++)
        {
          for (j=0; j<48; j++)
          {
            if (i<LEVEL_LEN)
            {
              if (level1[i][j]==1)
                vbSetObject(1023-objcnt++, OBJ_ON, j*8, 0, 216-(i*8)+scrollcnt, 13);
            }
          }
        }
        VIP_REGS[SPT3] = 1023;
        VIP_REGS[SPT2] = 1023-objcnt;
        WA[WRLD_BKGND].my=136-((scrollcnt/4)&7);
      }

      if ((scrollcnt+y)>score) //update score
      {
        score=scrollcnt+y;
        bgmap_print(4, 40, 2, itoa(score,10,1), 0);
        if (score==1669) //you win!
        {
          SND_REGS[WAVE1].SxINT=0x00;
          vbSetWorld(31, WRLD_ON|4, 0, 0, 0, 0, 0, 0, 383, 223); //score
          WA[30].head=WRLD_END;
          bgmap_print_blank(4, 40, 2, 6);
          bgmap_print(4, 16, 10, "Congratulations!!!", 0);
          bgmap_print(4, 19, 14, "Score:", 0);
          bgmap_print(4, 26, 14, itoa(score,10,1), 0);
          bgmap_print(4, 12, 20, "Press Start to play again", 0);
          while (!(vbReadPad() & K_STA));
          break;
        }
      }
      
      //draw current ball position
      WA[WRLD_BILL].gx=x-12;
      WA[WRLD_BILL].gy=200-y;
      
      if (y<0) //die
      {
        WA[WRLD_BILL].mx=72;
        if (y<(-24))
        {
          SND_REGS[WAVE1].SxINT=0x00;
          vbSetWorld(31, WRLD_ON|4, 0, 0, 0, 0, 0, 0, 383, 223); //score
          WA[30].head=WRLD_END;
          bgmap_print_blank(4, 40, 2, 6);
          bgmap_print(4, 19, 14, "Score:", 0);
          bgmap_print(4, 26, 14, itoa(score,10,1), 0);
          bgmap_print(4, 12, 20, "Press Start to play again", 0);
          while (!(vbReadPad() & K_STA));
          break;
        }
      }
      
      vbWaitFrame(8);
    }
  }
	return 0;
}
